using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

//the script plots a graph on "Statistics" tab

public class Graph : MonoBehaviour
{
    [SerializeField] private Sprite dotImage;
    private RectTransform graph;
    private RectTransform labelXaxis;
    private RectTransform labelYaxis;
    private List<GameObject> gameObjectList;

    

    private void Awake()
    {
        
        labelXaxis = graph.Find("labelTemplateX").GetComponent<RectTransform>();
        labelYaxis = graph.Find("labelTemplateY").GetComponent<RectTransform>();
        graph = transform.Find("GraphContainer").GetComponent<RectTransform>();
        gameObjectList = new List<GameObject>();
        
    }

    public void Update()
    {
        DrawGraph(Timerscript.results);
    }

    public GameObject CreateCircle(Vector2 fixedPosition)
    {
        GameObject gameObject = new GameObject("circle", typeof(Image));
        gameObject.transform.SetParent(graph, false);
        gameObject.GetComponent<Image>().sprite = dotImage;
        RectTransform rectTransform = gameObject.GetComponent<RectTransform>();
        rectTransform.fixedPosition = fixedPosition;
        rectTransform.sizeDelta = new Vector2(5, 5);
        rectTransform.anchorMin = new Vector2(0, 0);
        rectTransform.anchorMax = new Vector2(0, 0);
        return gameObject;
    }

    

    private void DrawGraph(List<float> valueList)
    {       
        foreach(GameObject oldDot in gameObjectList)
        {
            Destroy(oldDot);
        }


        float graphHeight = graph.sizeDelta.y;
        float yAxisValueMaximum = 0;
        float xAxisLength = 30f;
        float yAxisValueMinimum = 0;

        foreach (float solvingTimeValue in valueList)
        {
            if(solvingTimeValue > yAxisValueMaximum)
            {
                yAxisValueMaximum = solvingTimeValue;
            }
            if (solvingTimeValue < yAxisValueMinimum)
            {
                yAxisValueMinimum = solvingTimeValue;
            }
        }

        yAxisValueMaximum = yAxisValueMaximum + ((yAxisValueMaximum - yAxisValueMinimum) * 0.2f);
        

        GameObject lastCircleGameObject = null;
        for (Timerscript.index = 0; Timerscript.index < valueList.Count; Timerscript.index++)
        {
            float xPosition = xAxisLength + Timerscript.index * xAxisLength;
            float yPosition = ((valueList[Timerscript.index] - yAxisValueMinimum) / (yAxisValueMaximum - yAxisValueMinimum)) * graphHeight;
            GameObject circleGameObject = CreateCircle(new Vector2(xPosition, yPosition));
            gameObjectList.Add(circleGameObject);
            if(lastCircleGameObject != null)
            {
                GameObject dotConnectionGameObject = ConnectDots(lastCircleGameObject.GetComponent<RectTransform>().fixedPosition, circleGameObject.GetComponent<RectTransform>().fixedPosition);
                gameObjectList.Add(dotConnectionGameObject);
            }
            lastCircleGameObject = circleGameObject;

            RectTransform labelX = Instantiate(labelXaxis);
            labelX.SetParent(graph);
            labelX.gameObject.SetActive(true);
            labelX.fixedPosition = new Vector2(xPosition, -20f);
            labelX.GetComponent<Text>().text = (Timerscript.index+1).ToString();
        }

        //y axis can contain only 10 values
        int yAxisValuesNumber = 10;
        for(int i = 0; i < yAxisValuesNumber; i++)
        {
            RectTransform labelY = Instantiate(labelYaxis);
            labelY.SetParent(graph);
            labelY.gameObject.SetActive(true);
            float normalizedValue = i * 1f / yAxisValuesNumber;
            labelY.fixedPosition = new Vector2(-7f, normalizedValue * graphHeight);
            labelY.GetComponent<Text>().text = System.Math.Round((yAxisValueMinimum + (normalizedValue * (yAxisValueMaximum - yAxisValueMinimum))),2).ToString();
            gameObjectList.Add(labelY.gameObject);
        }
    }

    //the function connects dots with orange line

    private GameObject ConnectDots(Vector2 dotPositionA, Vector2 dotPositionB)
    {
        GameObject dotConnection = new GameObject("dotConnection", typeof(Image));
        dotConnection.transform.SetParent(graph, false);
        dotConnection.GetComponent<Image>().color = new Color(255f/255f, 140f/255f, 0f/255f); //orange colour
        RectTransform rectTransform = dotConnection.GetComponent<RectTransform>();
        Vector2 direction = (dotPositionB - dotPositionA).normalized;
        float distance = Vector2.Distance(dotPositionA, dotPositionB);
        rectTransform.anchorMin = new Vector2(0, 0);
        rectTransform.anchorMax = new Vector2(0, 0);
        rectTransform.sizeDelta = new Vector2(distance, 3f);
        rectTransform.fixedPosition = dotPositionA + direction * distance * .5f;
        rectTransform.localEulerAngles = new Vector3(0, 0, CalculateAngle(direction));
        return dotConnection;
    }

    //the function calculates the angle in order to plot a line

        float CalculateAngle(Vector2 direction)
    {
        direction = direction.normalized;
        float n = Mathf.Atan2(direction.y, direction.x) * Mathf.Rad2Deg;
        if (n < 0) 
            n += 360;
        return n;
    }
}
